/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { CLIENT_ALERT_TYPES } from '../../../common/constants/alerts';
import { apiService } from './utils';
import { ActionConnector } from '../alerts/alerts';

import { AlertsResult, MonitorIdParam } from '../actions/types';
import { ActionType, AsApiContract } from '../../../../triggers_actions_ui/public';
import { API_URLS } from '../../../common/constants';
import { Alert, AlertTypeParams } from '../../../../alerting/common';
import { AtomicStatusCheckParams } from '../../../common/runtime_types/alerts';

import { populateAlertActions, RuleAction } from './alert_actions';
import { Ping } from '../../../common/runtime_types/ping';

const UPTIME_AUTO_ALERT = 'UPTIME_AUTO';

export const fetchConnectors = async (): Promise<ActionConnector[]> => {
  const response = (await apiService.get(API_URLS.RULE_CONNECTORS)) as Array<
    AsApiContract<ActionConnector>
  >;
  return response.map(
    ({
      connector_type_id: actionTypeId,
      referenced_by_count: referencedByCount,
      is_preconfigured: isPreconfigured,
      is_missing_secrets: isMissingSecrets,
      ...res
    }) => ({
      ...res,
      actionTypeId,
      referencedByCount,
      isPreconfigured,
      isMissingSecrets,
    })
  );
};

export interface NewAlertParams extends AlertTypeParams {
  selectedMonitor: Ping;
  defaultActions: ActionConnector[];
}

type NewMonitorStatusAlert = Omit<
  Alert<AtomicStatusCheckParams>,
  | 'id'
  | 'createdBy'
  | 'updatedBy'
  | 'createdAt'
  | 'updatedAt'
  | 'apiKey'
  | 'apiKeyOwner'
  | 'muteAll'
  | 'mutedInstanceIds'
  | 'executionStatus'
  | 'alertTypeId'
  | 'notifyWhen'
  | 'actions'
> & {
  rule_type_id: Alert<AtomicStatusCheckParams>['alertTypeId'];
  notify_when: Alert<AtomicStatusCheckParams>['notifyWhen'];
  actions: RuleAction[];
};

export const createAlert = async ({
  defaultActions,
  monitorId,
  selectedMonitor,
}: NewAlertParams): Promise<Alert> => {
  const actions: RuleAction[] = populateAlertActions({
    defaultActions,
    selectedMonitor,
  });

  const data: NewMonitorStatusAlert = {
    actions,
    params: {
      numTimes: 1,
      timerangeUnit: 'm',
      timerangeCount: 1,
      shouldCheckStatus: true,
      shouldCheckAvailability: false,
      isAutoGenerated: true,
      search: `monitor.id : ${monitorId} `,
      filters: { 'url.port': [], 'observer.geo.name': [], 'monitor.type': [], tags: [] },
    },
    consumer: 'uptime',
    rule_type_id: CLIENT_ALERT_TYPES.MONITOR_STATUS,
    schedule: { interval: '1m' },
    notify_when: 'onActionGroupChange',
    tags: [UPTIME_AUTO_ALERT],
    name: `${selectedMonitor?.monitor.name || selectedMonitor?.url?.full}(Simple status alert)`,
    enabled: true,
    throttle: null,
  };

  return await apiService.post(API_URLS.CREATE_RULE, data);
};

export const fetchMonitorAlertRecords = async (): Promise<AlertsResult> => {
  const data = {
    page: 1,
    per_page: 500,
    filter: 'alert.attributes.alertTypeId:(xpack.uptime.alerts.monitorStatus)',
    default_search_operator: 'AND',
    sort_field: 'name.keyword',
    sort_order: 'asc',
    search_fields: ['name', 'tags'],
    search: 'UPTIME_AUTO',
  };
  return await apiService.get(API_URLS.RULES_FIND, data);
};

export const fetchAlertRecords = async ({
  monitorId,
}: MonitorIdParam): Promise<Alert<NewAlertParams>> => {
  const data = {
    page: 1,
    per_page: 500,
    filter: 'alert.attributes.alertTypeId:(xpack.uptime.alerts.durationAnomaly)',
    default_search_operator: 'AND',
    sort_field: 'name.keyword',
    sort_order: 'asc',
  };
  const alerts = await apiService.get<{ data: Array<Alert<NewAlertParams>> }>(
    API_URLS.RULES_FIND,
    data
  );
  return alerts.data.find((alert) => alert.params.monitorId === monitorId) as Alert<NewAlertParams>;
};

export const disableAlertById = async ({ alertId }: { alertId: string }) => {
  return await apiService.delete(API_URLS.DELETE_RULE + alertId);
};

export const fetchActionTypes = async (): Promise<ActionType[]> => {
  const response = (await apiService.get(API_URLS.CONNECTOR_TYPES)) as Array<
    AsApiContract<ActionType>
  >;
  return response.map<ActionType>(
    ({
      enabled_in_config: enabledInConfig,
      enabled_in_license: enabledInLicense,
      minimum_license_required: minimumLicenseRequired,
      ...res
    }: AsApiContract<ActionType>) => ({
      ...res,
      enabledInConfig,
      enabledInLicense,
      minimumLicenseRequired,
    })
  );
};
